% Note this file shadows the class @test_shadow.  The class will
% take precedence: i.e., `doctest test_shadow` will test the class.
% You can always override with `doctest test_shadow.m`.
%
% >> 2 + 2
% ans = 4
% >> 3 + 3
% ans = 6
%
%
% Note: in Matlab, help('test_shadow.m') returns the help for
% '@test_shadow' even when the .m if explicitly given.  So this
% test will never run on Matlab.

not_the_class = 42
