function test_ellipsis()
% >> '...'   % doctest: -ELLIPSIS
%
% ...
%
%
% >> 1 + 2
%
% ...
%
%
% >> 1 + 2   % doctest: -ELLIPSIS   % doctest: +XFAIL
%
% ...
%
%
% >> 1 + 2   % doctest: -ELLIPSIS
%
% ans = 3
%
